C3 metric interceptors for librdkafka based clients
===================================================

Provides a plugin library for librdkafka based clients
that implements the client-side C3 metrics reporter
using consumer and producer interceptors.

The same library can be used for both the consumer
and producer, using the same configuration.

To load the library set the librdkafka configuration
property `plugin.library.paths` to the path of this
library (including filename).



## Configuration properties

Interceptor configuration properties are set on the intercepted client's
configuration object with the "confluent.monitoring.interceptor." prefixed
to any configuration property.

Standard librdkafka configuration properties with the above prefix are passed
through as-is to the interceptor's producer config.

To load the plugin, append the plugin library path to `plugin.library.paths`:
 * `plugin.library.paths=/opt/confluent/librdkafka-plugins/monitoring-interceptor.so`

The following properties configures the interceptor itself:
 * `confluent.monitoring.interceptor.publishMs` - Metrics publishing interval (ms)
 * `confluent.monitoring.interceptor.sessionDurationMs` - Metric session duration (ms)
 * `confluent.monitoring.interceptor.topic` - C3 topic to produce metrics to
 * `confluent.monitoring.interceptor.icdebug` - Enable ("true") interceptor debugging (to stderr)


## Example configuration

```
# Intercepted client's configuration
bootstrap.servers=broker1,broker2
...

# Interceptor configuration
plugin.library.paths=/opt/confluent/librdkafka-plugins/monitoring-interceptor.so
confluent.monitoring.interceptor.publishms=5000
confluent.monitoring.interceptor.bootstrap.servers=myc3broker1,myc3broker2
...
```



## Build

    # From the top-level directory
    ./configure
    make


## Test

    make tests



## Tools

A tool for printing the monitoring protobuf messages is available
in the [tools](tools) directory.


    make tools

Generate nanopb protobuf files (nanopb-gen)
===========================================

Using the Protocol Buffers compiler (protoc)
--------------------------------------------
The nanopb generator is implemented as a plugin for the Google's own protoc
compiler. This has the advantage that there is no need to reimplement the
basic parsing of .proto files. However, it does mean that you need the
Google's protobuf library in order to run the generator.

If you have downloaded a binary package for nanopb (either Windows, Linux or
Mac OS X version), the 'protoc' binary is included in the 'generator-bin'
folder. In this case, you are ready to go. Simply run this command:

    generator-bin/protoc --nanopb_out=. myprotocol.proto

However, if you are using a git checkout or a plain source distribution, you
need to provide your own version of protoc and the Google's protobuf library.
On Linux, the necessary packages are protobuf-compiler and python-protobuf.
On Windows, you can either build Google's protobuf library from source or use
one of the binary distributions of it. In either case, if you use a separate
protoc, you need to manually give the path to nanopb generator:

    protoc --plugin=protoc-gen-nanopb=nanopb/generator/protoc-gen-nanopb ...